function LensReport_DualBand(varargin)
% fn = 'C:\Users\Dan\Documents\SWIR Zoom\Color Correction\vsTest.len';
global filepath

if ischar(filepath)
    [fn0,pn0] = uigetfile({'*.seq;*.len', 'CodeV Lens Files (*.seq,*.len)'},'Choose a lens file to visualize',filepath);
else
    try
        un = getenv('USERNAME');
        [fn0,pn0] = uigetfile({'*.seq;*.len', 'CodeV Lens Files (*.seq,*.len)'},'Choose a lens file to visualize',['C:\Users\' un '\Documents\']);
    catch wrongpath
        [fn0,pn0] = uigetfile({'*.seq;*.len', 'CodeV Lens Files (*.seq,*.len)'},'Choose a lens file to visualize');
    end
end

if(fn0)==0
    disp('No file selected');
    return;
end
filepath=pn0;

fn = [pn0,fn0];
[~,~,lensfiletype] = fileparts(fn);

set(0,'Units','pixels');

[curpath,~,~] = fileparts(mfilename('fullpath'));
curpath = [curpath '\'];
macroname = ['"' curpath 'MatlabOutput_DualBand.seq"'];

opengl software;

%% CodeV
try
    % start instance of CodeV in the background and institute COM link
    cvon;
    global CodeV; % activeX server variable
    
    % CodeV shortcut functions
    sendcmd = @(cmd) invoke(CodeV, 'command', cmd); % send a command to CodeV command line
    bufput = @(b,i,j,str) invoke(CodeV, 'command', ['buf put b' int2str(b) ' i' int2str(i) ' j' int2str(j) ' (' str ')']); % put value defined by "str" into buffer b, row i, column j
    bufdel = @(b) invoke(CodeV, 'command', ['buf del b' int2str(b)]); % delete buffer b
    buffnd = @(b,exp) invoke(CodeV, 'command', ['buf fnd b' int2str(b) ' ' exp]); % find expression "exp" in buffer "b" and move cursor position to that point
    
    % pre-defined buffer numbers
    buf0 = 100;
    buf1 = 101;
    buf2 = 102;
    buf3 = 103;
    
    tic; disp('Opening Lens...');
    % open lens file
    if strcmpi(lensfiletype, '.seq');
        sendcmd(['IN "' fn '"']);
    elseif strcmpi(lensfiletype, '.len');
        sendcmd(['RES "' fn '"']);
    else
        error('Lens File Type Not Recognized');
    end
    
    % get initial data for dual band
    [s,f,w,z] = cvnum(); % number of [surfaces, fields, wavelengths, zoom positions] in lens
    wref = zeros(1,z); % reference wavelength number
    wav = zeros(1,w);
    wtw = zeros(w,z);
    for wl = 1:w
        wav(wl) = cvevadbl(['wl w' int2str(wl)]);
        for zl = 1:z
            wtw(wl,zl) = cvevadbl(['wtw w' int2str(wl) ' z' int2str(zl)]);
        end
    end
    for zl = 1:z
        wref(zl) = cvevadbl(['ref z' int2str(zl)]);
    end
    
    % input box for dual band selection
    % find defaults
    dbnum = 2;
    dbname = {'VIS';'SWIR'};
    dbcutoff = 800; % cutoff wavelength in nm
    dbwavi = cell(2,1);
    dbwavi{1} = find(wav<=dbcutoff);
    dbwavi{2} = find(wav>dbcutoff);
    dbwtwmean = zeros(dbnum,z);
    bufdel(buf1);
    eflwz = zeros(w,z);
    for zi = 1:z
        for di = 1:dbnum
            dbwtwmean(di,zi) = mean(wtw(dbwavi{di},zi));
        end
        for wi = 1:w
            bufput(buf1,wi,zi,['efy s1..i z' int2str(zi) ' w' int2str(wi)]);
        end
    end
    [~, eflwz] = invoke(CodeV, 'BufferToArray', 1, w, 1, z, buf1, eflwz);
    [~,dbdomb] = max(dbwtwmean,[],1); % dominant band for each zoom
    dbz = false([dbnum,z]);
    for zi = 1:z
        dbz(dbdomb(zi),zi) = true;
    end
    dbwref = zeros(dbnum,1); % dominant reference wavelength index for each band
    for di = 1:dbnum
        dbwref(di) = mode(wref(dbdomb==di));
    end
    
    % make figure
    dbfig = initfig('Dual Band',6);
    opos = get(dbfig,'position');
    hwid = [120 100 60 60 150 60 60]; % column widths [cutoff, wavelength list, band 1 reference wav, band 2 reference wav, zoom titles, band 1 zoom, band 2 zoom]
    hpos = [0,cumsum(hwid)]+10; % horizontal positions
    nw = max(hpos)+10; % new figure width
    nh = 400; % new figure height
    set(dbfig,'position',[opos(1),opos(2)+opos(4)-nh,nw,nh],'ToolBar','none');
    vposw = linspace(50,nh-70,w);
    vposz = linspace(nh-70,50,z);
    vhet = 20;
    dbhanw = zeros(w,2); % wavelength control handles
    dbhanz = zeros(z,3); % zoom control handles [zoom title, band 1 checkboxes, band 2 checkboxes]
    dbhant = zeros(dbnum,3); % band title handles, 1 row for each band, column 1 for edit boxes on slider, col 2 for reference wavelength titles, col 3 for zoom band titles
    uicontrol('style','text','string','Cutoff(nm)','units','pixels','position',[hpos(1) nh-20 diff(hpos(1:2)) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    cobox = uicontrol('style','edit','string',num2str(dbcutoff),'units','pixels','position',[hpos(1) nh-40 diff(hpos(1:2)) 20],'parent',dbfig,'fontsize',12,'callback',@coboxcall);
    slidebuff = 0;
    coslider = uicontrol('style','slider','units','pixels','position',[hpos(2)-20 min(vposw)-slidebuff 15 max(vposw)-min(vposw)+vhet+slidebuff*2],'parent',dbfig,'max',-min(wav)-1,'min',-max(wav)+1,'value',-dbcutoff,'callback',@coslidercall);
    db1vpos = min(vposw)+(max(vposw)-min(vposw))*(1-((dbcutoff+min(wav))/2-min(wav))/(max(wav)-min(wav)))-10;
    db2vpos = min(vposw)+(max(vposw)-min(vposw))*(1-((dbcutoff+max(wav))/2-min(wav))/(max(wav)-min(wav)))-10;
    dbhant(1,1) = uicontrol('style','edit','string',dbname{1},'units','pixels','position',[hpos(1) db1vpos diff(hpos(1:2))-30 20],'parent',dbfig,'fontsize',12);
    dbhant(2,1) = uicontrol('style','edit','string',dbname{2},'units','pixels','position',[hpos(1) db2vpos diff(hpos(1:2))-30 20],'parent',dbfig,'fontsize',12);
    set(dbhant(:,1),'callback',@bandnamecall);
    uicontrol('style','text','string',{'Wavelength';'(nm)'},'units','pixels','position',[hpos(2) nh-40 diff(hpos(2:3)) 40],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    bg1 = uibuttongroup('units','pixels','position',[hpos(3) 1 diff(hpos(3:4)) nh],'backgroundcolor','w','bordertype','none','parent',dbfig);
    bg2 = uibuttongroup('units','pixels','position',[hpos(4) 1 diff(hpos(4:5)) nh],'backgroundcolor','w','bordertype','none','parent',dbfig);
    set([bg1,bg2],'SelectionChangeFcn',@dbwrefcall);
    uicontrol('style','text','string','Reference','units','pixels','position',[hpos(3) nh-20 diff(hpos([3,5])) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    dbhant(1,2) = uicontrol('style','text','string',dbname{1},'units','pixels','position',[hpos(3) nh-40 diff(hpos(3:4)) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    dbhant(2,2) = uicontrol('style','text','string',dbname{2},'units','pixels','position',[hpos(4) nh-40 diff(hpos(4:5)) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    uicontrol('style','text','string',{'Zoom';'Position'},'units','pixels','position',[hpos(5) nh-40 diff(hpos(5:6)) 40],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    dbhant(1,3) = uicontrol('style','text','string',dbname{1},'units','pixels','position',[hpos(6) nh-40 diff(hpos(6:7)) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    dbhant(2,3) = uicontrol('style','text','string',dbname{2},'units','pixels','position',[hpos(7) nh-40 diff(hpos(7:8)) 20],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
    uicontrol('style','pushbutton','string','Cancel','units','pixels','position',[nw-310 5 150 40],'parent',dbfig,'fontsize',12,'callback',@dbcancall);
    uicontrol('style','pushbutton','string','Continue','units','pixels','position',[nw-155 5 150 40],'parent',dbfig,'fontsize',12,'callback',@dbokcall);
    for wd = 1:w
        uicontrol('style','text','string',sprintf('%0.0f',wav(wd)),'units','pixels','position',[hpos(2) vposw(wd) diff(hpos(2:3)) vhet],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
        dbhanw(wd,1) = uicontrol('style','radiobutton','units','pixels','position',[round(hwid(3)/2)-7 vposw(wd)+3 14 14],'parent',bg1,'backgroundcolor','w');
        dbhanw(wd,2) = uicontrol('style','radiobutton','units','pixels','position',[round(hwid(4)/2)-7 vposw(wd)+3 14 14],'parent',bg2,'backgroundcolor','w');
    end
    for zd0 = 1:z
        dbhanz(zd0,1) = uicontrol('style','text','string',sprintf('Z%0.0f, %0.0fmm',[0,0]),'units','pixels','position',[hpos(5) vposz(zd0) diff(hpos(5:6)) vhet],'parent',dbfig,'fontsize',12,'backgroundcolor','w');
        dbhanz(zd0,2) = uicontrol('style','checkbox','units','pixels','position',[round(mean(hpos(6:7)))-7 vposz(zd0)+3 14 14],'parent',dbfig);
        dbhanz(zd0,3) = uicontrol('style','checkbox','units','pixels','position',[round(mean(hpos(7:8)))-7 vposz(zd0)+3 14 14],'parent',dbfig);
    end
    dbhanztrunc = dbhanz(:,2:3);
    set(dbhanztrunc,'callback',@dbzoomcall);
    dbupdate(0);
    dbupdate(1);
    dbupdatez();
    canceldb = 1;
    waitfor(dbfig);
%     canceldb = 1; % for debugging
    if canceldb==1;
        cvoff;
        return;
    end
    dbzoomi = {find(dbz(1,:)),find(dbz(2,:))};
    
    inpt = inputdlg({'Number of rays to trace for RIM plots:','Number of L/mm sample points for MTF:','Scale for RIM plots in mm:'},'Input',1,{'21','70','0'});
    rimpts = floor(str2double(inpt{1})/2)*2+1; % made to be odd to include 0 relative pupil height
    freqpts = round(str2double(inpt{2}));
    rngr = str2double(inpt{3});
    
    if ~( isnumeric(rimpts) && isnumeric(freqpts) )
        error('Input must be numeric.');
    elseif (rimpts<=1) || (freqpts<=1)
        error('Number of points must be greater than 1.');
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%
    tic; disp('Obtaining Lens Data...');
    % get lens data from CodeV
    [s,f,w,z] = cvnum(); % number of [surfaces, fields, wavelengths, zoom positions] in lens
    invoke(CodeV,'SetCommandTimeout',round(6000*rimpts/10*z/5*f/3*w/5));
    surfdata = cvsd(); % extract lens manager data
    surfnamesraw = cellfun(@(raw) textscan(raw,'%u%s'),surfdata(:,1),'UniformOutput',false);
    surfnums = cell(size(surfnamesraw)); % surface numbers in CodeV (matches up with row index if all surfaces are extracted
    surfnames = surfnums; % surface names if they exist
    bufdel(buf0);
    for ij = 1:s
        surfnums{ij} = surfnamesraw{ij}{1};
        surfnames{ij} = surfnamesraw{ij}{2}{1};
        bufput(buf0,ij,1,['zpr s' int2str(ij) ' thi']); % zoomed parameter flag
        tempchar = sendcmd(['EVA(GLA S' num2str(ij) ')']); % get raw glass name output from surface si
        qpos = strfind(tempchar,'"');
        surfdata{ij,6} = tempchar(qpos(1)+1:qpos(2)-1);
        tempchar = sendcmd(['EVA(GLA S' num2str(ij) ' CAT)']); % get glass catalog name from surface si
        qpos = strfind(tempchar,'"');
        surfdata{ij,9} = tempchar(qpos(1)+1:qpos(2)-1);
        surfdata{ij,8} = cvevadbl(['IND S' num2str(ij)]); % get glass index
    end
    zpr = zeros(s,1);
    [~, zpr] = invoke(CodeV, 'BufferToArray', 1, s, 1, 1, buf0, zpr);
    surfnums = cell2mat(surfnums);
    disp(num2str(toc));
    
    % get wavelength data
    wref = cvevadbl('REF'); % reference wavelength number
    wav = zeros(1,w);
    for wl = 1:w
        wav(wl) = cvevadbl(['wl w' int2str(wl)]);
    end
    
    % define zoom groups
    zpr = find(zpr);
    if isempty(zpr)
        gbnums = [1,s];
    else
        gbnums = zeros(numel(zpr),2);
        gbnums(:,2) = zpr;
        gbnums(1,1) = 1;
        gbnums(end+1,2) = s;
        gbnums(2:end,1) = zpr+1;
        gbnums(diff(gbnums,1,2)<2,:) = [];
        for gk = 1:size(gbnums,1)
            eltsurf = gbnums(gk,1)-1+find(cell2mat(surfdata(gbnums(gk,1):(gbnums(gk,2)-1),8))>1.01);
            gbnums(gk,2) = max(eltsurf)+1;
            if ~any(cell2mat(surfdata(gbnums(gk,1):gbnums(gk,2),8))>1.01)
                gbnums(gk,:) = [];
            end
        end
    end
    numgrps = size(gbnums,1);
    sdim = [floor(numgrps/2),ceil(numgrps/2)];
    
    % load buffer 60 with group boundary surfaces
    bufdel(60);
    a2b = invoke(CodeV, 'ArrayToBuffer', numgrps, 2, 60, gbnums);
    if a2b~=0
        error('Trouble Populating CV Buffer');
    end
    
    % load buffer 59 with dual band info
    bufin = zeros(3*dbnum,max(max(cellfun(@numel,dbzoomi)),2));
    for bi = 1:dbnum
        ro = (bi-1)*3; % row offset
        bufin(1+ro,1) = numel(dbzoomi{bi});
        bufin(1+ro,2) = dbwref(bi);
        bufin(2+ro,1:numel(dbzoomi{bi})) = dbzoomi{bi};
        bufin(3+ro,1) = min(dbwavi{bi});
        bufin(3+ro,2) = max(dbwavi{bi});
    end
    a2b = invoke(CodeV, 'ArrayToBuffer', size(bufin,1), size(bufin,2), 59, bufin);
    if a2b~=0
        error('Trouble Populating CV Buffer');
    end
    
    tic; disp('Running Data Generation Macro...');
    % run macro
    macmd = ['in ' macroname ' ' int2str(rimpts) ' ' int2str(numgrps)]; %sprintf(' %1.0f',[gbnums(:,1);gbnums(end,2)])];
    sendcmd(macmd);
    disp(num2str(toc));
    
    tic; disp('Obtaining Zoomed Data...');
    % get zoomed thickness and EFL
    zthik = zeros(s,z); % zoomed surface thicknesses
    zefl = zeros(1,z); % EFL at each zoom position
    [~, zthik] = invoke(CodeV, 'BufferToArray', 1, s, 1, z, 61, zthik);
    [~, zefl] = invoke(CodeV, 'BufferToArray', 1, 1, 1, z, 62, zefl);
    disp(num2str(toc));
    
    % input box for MTF and RIM plots
    qfig = initfig('Plot Choice',6);
    opos = get(qfig,'position');
    nw = 370*dbnum; % new figure width
    nh = 400; % new figure height
    ph = nh-50; % uipanel height in pixels
    set(qfig,'position',[opos(1),opos(2)+opos(4)-nh,nw,nh],'ToolBar','none');
    vhet = 20;
    qhan = cell(dbnum,1);
    dbzrim = qhan;
    maxfreq = qhan;
    pan = zeros(dbnum,1);
    pancol = [0 0 1; 1 0 0];
    for bi = 1:dbnum
        bkcol = min(1,pancol(bi,:)+0.9);
        pan(bi) = uipanel(qfig,'units','pixels','position',[(bi-1)*nw/dbnum nh-ph nw/dbnum ph],'Title',dbname{bi},'backgroundcolor',bkcol,'foregroundcolor',pancol(bi,:),'fontsize',12,'fontweight','bold');
        zlist = dbzoomi{bi};
        zefl1 = eflwz(dbwref(bi),zlist);
        z1 = numel(zefl1);
        vpos = linspace(ph-50,5,z1);
        qhan{bi} = zeros(z1,3);
        [~,midz] = min(abs(zefl1-mean([max(zefl1),min(zefl1)])));
        [~,minz] = min(zefl1);
        [~,maxz] = max(zefl1);
        dbzrim{bi} = [minz,midz,maxz]; % default zoom positions to plot
        maxfreq{bi} = linspace(100,50,z1).*2./bi;
        uicontrol(pan(bi),'style','text','string','Plot?','units','pixels','position',[180-15 ph-20 40 20],'parent',pan(bi),'fontsize',12,'backgroundcolor',bkcol);
        uicontrol(pan(bi),'style','text','string','MTF Max Freq.','units','pixels','position',[240 ph-20 120 20],'parent',pan(bi),'fontsize',12,'backgroundcolor',bkcol);
        for zq = 1:z1
            if ismember(zq,dbzrim{bi})
                enbl = 'on';
            else
                enbl = 'off';
            end
            qhan{bi}(zq,3) = uicontrol(pan(bi),'style','text','string',sprintf('Z%0.0f, EFL %0.0f mm',[zlist(zq),zefl1(zq)]),'units','pixels','position',[10 vpos(zq) 135 vhet],'enable',enbl,'fontsize',12,'backgroundcolor',bkcol);
            qhan{bi}(zq,1) = uicontrol(pan(bi),'style','checkbox','units','pixels','position',[180 vpos(zq)+3 14 14],'backgroundcolor','w');
            qhan{bi}(zq,2) = uicontrol(pan(bi),'style','edit','units','pixels','position',[275 vpos(zq) 50 vhet],'enable',enbl,'fontsize',12,'backgroundcolor','w');
            set(qhan{bi}(zq,1),'callback',{@callchk,qhan{bi}(zq,2:3)});
            set(qhan{bi}(zq,2),'string',num2str(round(maxfreq{bi}(zq))));
            if ismember(zq,dbzrim{bi})
                set(qhan{bi}(zq,1),'Value',get(qhan{bi}(zq,1),'Max'));
            else
                set(qhan{bi}(zq,1),'Value',get(qhan{bi}(zq,1),'Min'));
            end
        end
    end
    qbut(1) = uicontrol('style','pushbutton','string','Continue','units','pixels','position',[nw-105,5,100,40],'callback',{@qin,qhan},'fontsize',12);
    qbut(2) = uicontrol('style','pushbutton','string','Cancel','units','pixels','position',[5 5 100 40],'callback',@cancel,'fontsize',12);
    cancelq = 1;
    
    waitfor(qfig);
    
    if cancelq==1;
        cvoff;
        return;
    end
    
    tic; disp('Obtaining Element Data...');
    % element properties
    eltinds = find(cell2mat(surfdata(:,8))>1.1); % row indices of first surface of each element
    eltefl = zeros(size(eltinds)); % EFLs of each element
    bufdel(buf0);
    for eip = 1:numel(eltinds)
        bufput(buf0, eip, 1, ['EFY S' int2str(surfnums(eltinds(eip))) '..' int2str(surfnums(eltinds(eip))+1)]);
    end
    [~, eltefl] = invoke(CodeV, 'BufferToArray', 1, numel(eltinds), 1, 1, buf0, eltefl);
    eltpow = 1000./eltefl; % power of each element in m^-1
    disp(num2str(toc));
    
    tic; disp('Gathering Macro Data...');
    % reference ray (R1 center, R2 positive Y, R3 negative Y) positions at each surface
    rayz0 = zeros([s,z*f*3*w]);
    rayy0 = rayz0;
    dcz = zeros([z,f]); % incident Z direction cosines
    dcy = dcz; % incident Y direction cosines
    [~,rayz0] = invoke(CodeV, 'BufferToArray', 1, size(rayz0,1), 1, size(rayz0,2), 63, rayz0);
    [~,rayy0] = invoke(CodeV, 'BufferToArray', 1, size(rayy0,1), 1, size(rayy0,2), 64, rayy0);
    rayz = reshape(rayz0,[s,z,f,3,w]); % ray z position relative to vertex of each surface
    rayy = reshape(rayy0,[s,z,f,3,w]); % ray y position relative to vertex of each surface
    [~,dcz] = invoke(CodeV, 'BufferToArray', 1, size(dcz,1), 1, size(dcz,2), 65, dcz);
    [~,dcy] = invoke(CodeV, 'BufferToArray', 1, size(dcy,1), 1, size(dcy,2), 66, dcy);
%     disp(num2str(toc));
    
%     tic; disp('Obtaining RIM Plot Data...');
    % extract data for RIM plots
    pupx = linspace(-1,1,rimpts).'; % x relative pupil -1 to 1
    pupy = linspace(-1,1,rimpts).'; % y relative pupil -1 to 1
    rimx0 = zeros([rimpts,w*f*z]); % saggital ray "height" on image surface
    rimy0 = rimx0; % tangential ray height on image surface
    [~,rimx0] = invoke(CodeV, 'BufferToArray', 1, size(rimx0,1), 1, size(rimx0,2), 68, rimx0);
    [~,rimy0] = invoke(CodeV, 'BufferToArray', 1, size(rimy0,1), 1, size(rimy0,2), 67, rimy0);
    rimx = reshape(rimx0,[rimpts,w,f,z]);
    rimy = reshape(rimy0,[rimpts,w,f,z]);
%     disp(num2str(toc));

%     tic; disp('Obtaining Vignetting Data...');
    % extract vignetting data
    pupx = linspace(-1,1,rimpts).'; % x relative pupil -1 to 1
    pupy = linspace(-1,1,rimpts).'; % y relative pupil -1 to 1
    vuy = zeros(f,z); % Y upper vignetting
    vly = vuy; % Y lower vignetting
    vux = vuy; % X upper vignetting
    vlx = vuy; % X lower vignetting
    [~,vuy] = invoke(CodeV, 'BufferToArray', 1, size(vuy,1), 1, size(vuy,2), 86, vuy);
    [~,vly] = invoke(CodeV, 'BufferToArray', 1, size(vly,1), 1, size(vly,2), 87, vly);
    [~,vux] = invoke(CodeV, 'BufferToArray', 1, size(vux,1), 1, size(vux,2), 88, vux);
    [~,vlx] = invoke(CodeV, 'BufferToArray', 1, size(vlx,1), 1, size(vlx,2), 89, vlx);
    vig = cat(3,vuy,vly,vux,vlx);
    vig = max(0,vig);
%     disp(num2str(toc));
    
    zfno = zeros(dbnum,max(cellfun(@numel,dbzoomi)));
    [~,zfno] = invoke(CodeV, 'BufferToArray', 1, size(zfno,1), 1, size(zfno,2), 78, zfno);
    
    tic; disp('Obtaining THO Data...');
    % extract third order aberration data
    dbtho = cell(dbnum,1);
    for bi = 1:dbnum
        thosa = zeros(numgrps,numel(dbzoomi{bi}));
        thotc = thosa;
        thota = thosa;
        thopt = thosa;
        thods = thosa;
        thoax = thosa;
        thola = thosa;
        [~,thosa] = invoke(CodeV, 'BufferToArray', 1, size(thosa,1), 1, size(thosa,2), 69+10*(bi-1), thosa);
        [~,thotc] = invoke(CodeV, 'BufferToArray', 1, size(thotc,1), 1, size(thotc,2), 70+10*(bi-1), thotc);
        [~,thota] = invoke(CodeV, 'BufferToArray', 1, size(thota,1), 1, size(thota,2), 71+10*(bi-1), thota);
        [~,thopt] = invoke(CodeV, 'BufferToArray', 1, size(thopt,1), 1, size(thopt,2), 72+10*(bi-1), thopt);
        [~,thods] = invoke(CodeV, 'BufferToArray', 1, size(thods,1), 1, size(thods,2), 73+10*(bi-1), thods);
        [~,thoax] = invoke(CodeV, 'BufferToArray', 1, size(thoax,1), 1, size(thoax,2), 74+10*(bi-1), thoax);
        [~,thola] = invoke(CodeV, 'BufferToArray', 1, size(thola,1), 1, size(thola,2), 75+10*(bi-1), thola);
        dbtho{bi} = cat(3,thosa,thotc,thota,thopt,thods,thoax,thola);
    end
    disp(num2str(toc));
    
    % extract wavelength dependent surface index data
    index_w = zeros(s,w);
    [~,index_w] = invoke(CodeV, 'BufferToArray', 1, size(index_w,1), 1, size(index_w,2), 76, index_w);
    
    % get group EFLs per wavelength
    grpefl = zeros(size(gbnums,1),w); % group EFLs
    [~, grpefl] = invoke(CodeV, 'BufferToArray', 1, size(grpefl,1), 1, size(grpefl,2), 77, grpefl);
    
%     error('debug');
    tic; disp('Obtaining MTF Data...');
    % extract MTF data
    mtfdata = cell(dbnum,1);
    for bi = 1:dbnum
        zlist = dbzoomi{bi};
        zrim = dbzrim{bi};
        wlist = dbwavi{bi};
        zstr = '';
        wstr = '';
        for zm = 1:numel(zlist) %z
            wstr = [wstr 'REF Z' int2str(zlist(zm)) ' ' int2str(dbwref(bi)) '; WTW Z' int2str(zlist(zm)) ' W1..' int2str(w) ' 0; '];
            for wc = 1:numel(wlist)
                wstr = [wstr 'WTW Z' int2str(zlist(zm)) ' W' int2str(wlist(wc)) ' ' num2str(wtw(wlist(wc),zlist(zm))) '; '];
            end
            if ismember(zm,zrim)
                zstr = [zstr, 'mfr z' int2str(zlist(zm)) ' ' num2str(maxfreq{bi}(zm)*(1+1/(freqpts-1))) '; ifr z' int2str(zlist(zm)) ' ' num2str(maxfreq{bi}(zm)/(freqpts-1)*.9999999) '; '];
            else
                zstr = [zstr, 'mfr z' int2str(zlist(zm)) ' 1; ifr z' int2str(zlist(zm)) ' 1; '];
            end
        end
        bufdel(0);
        sendcmd(wstr);
        sendcmd(['buf; mtf; geo no; cht fre n; ' zstr 'go; buf n;']);
        % read mtf data from buffer
        mtfdata{bi} = zeros(freqpts,6,f,numel(zrim)); % MTF tabulated data
        bufdel(buf0);
        buffnd(0,'"L/MM"');
        lmln = zeros(z*f,1);
        for lm = 1:z*f
            bufput(buf0, lm, 1, 'buf.i b0');
            sendcmd('buf fnd fwd');
        end
        [~, lmln] = invoke(CodeV, 'BufferToArray', 1, size(lmln,1), 1, size(lmln,2), buf0, lmln);
        for zf = 1:numel(zrim)
            for ff = 1:f
                lmind = (zlist(zrim(zf))-1)*f + ff;
                if ff==1
                    mtf0 = zeros(freqpts,4);
                else
                    mtf0 = zeros(freqpts,6);
                end
                [~, mtf0] = invoke(CodeV, 'BufferToArray', lmln(lmind)+2, lmln(lmind)+2+size(mtf0,1)-1, 1, size(mtf0,2), 0, mtf0);
                if ff==1
                    mtf0 = mtf0(:,[1,2,3,3,4,4]);
                end
                mtfdata{bi}(:,:,ff,zf) = mtf0;
            end
        end
    end
    disp(num2str(toc));
    
    % other data
    stopsurf = cvevadbl('STO'); % stop surface number
    yan = zeros(1,f);
    for fa = 1:f
        yan(fa) = cvevadbl(['yan f' int2str(fa)]);
    end
    
catch err
    cvoff;
    rethrow(err);
end
cvoff;

%% Initial Data Processing
maxsd = max(cell2mat(surfdata(:,5)));

gthik = cumsum(zthik,1);
gthik = [zeros(1,z);gthik];

rayz0 = -0.1*max(gthik(:));
rayy0 = rayy(1,:,:,:,:)+(rayz0-rayz(1,:,:,:,:)).*repmat(reshape(dcy./dcz,[1,size(dcy)]),[1,1,1,3,w]);

stopind = find(surfnums==stopsurf);
stopsd = surfdata{stopind,5};
stoppos = gthik(stopind,:);
stopedge = 0.4; % stop width to draw in fraction of maximum lens semi diameter

glasinds = unique(cell2mat(surfdata(eltinds,8)));

matrep = @(mat) repmat(mat,10,1);
colorscale = 1.7; % >= 1
ncolors = jet(numel(glasinds))./colorscale + (1-1/colorscale);
fcolors = {'r';'g';'b';'m';'y';'c';'k'};
fcolors = matrep(fcolors);
fcolorsrgb = [1 0 0;
              0 0.5 0;
              0 0 1;
              1 0 1;
              1 1 0;
              0 1 1;
              0 0 0];
fcolorsrgb = matrep(fcolorsrgb);
wcolors = [1 0 0;...
           0 0.5 0;...
           0 1 1;...
           0 0 1;...
           1 0 1;...
           0 0 0];
wcolors = matrep(wcolors);
gcolors = get(0,'DefaultAxesColorOrder');
gcolors = matrep(gcolors);
gcolorscale = 3; % >= 1
gcolorss = gcolors./gcolorscale + (1-1/gcolorscale);

firelt = zeros(size(glasinds));
for ii = 1:numel(glasinds);
    firelt(ii) = find(cell2mat(surfdata(eltinds,8))==glasinds(ii),1);
end
[~,sorder] = sort(cell2mat(surfdata(eltinds(firelt),8)));
firelts = firelt(sorder);

wlong = cellfun(@min,dbwavi);
wshort = cellfun(@max,dbwavi);
wtarg = mean(wav([wshort,wlong]),2);
wmid = zeros(dbnum,1);
for bi = 1:dbnum
    [~,wmid0] = min(abs(wav(dbwavi{bi})-wtarg(bi)));
    wmid(bi) = dbwavi{bi}(wmid0);
end
abbe = (index_w(:,wmid)-1)./(index_w(:,wshort)-index_w(:,wlong)); % [#glasses, #bands]
partial = (index_w(:,wmid)-index_w(:,wlong))./(index_w(:,wshort)-index_w(:,wlong));

numpts = 40;
eltz = zeros(2*numpts,numel(eltinds));
elty = eltz;
for eii = 1:numel(eltinds)
    [eltz(:,eii),elty(:,eii)] = eltpts(eltinds(eii),surfdata,numpts,0);
end

%% Lens Drawing
for bi = 1:2
    lensdrawing( bi, dbzoomi{bi}, dbwavi{bi}, dbwref(bi), eflwz(dbwref(bi),dbzoomi{bi}) );
end

    function lensdrawing(band,zlist,wlist,wref,zefl)
        z0 = numel(zlist);
        w0 = numel(wlist);
        f1h = initfig(['Lens Drawing ' int2str(band)],band,dbname{band});
        ax1 = axes('units','normal','position',[0.05 0.15 .7 .8]);
        axis equal;
        set(ax1,'fontweight','bold');
        elth = zeros(numel(eltinds),z0);
        offset = 2.2*maxsd.*((1:z0).'-1);
        [rayymxw,rayymxi] = max(rayy(:,zlist,:,:,wlist),[],5);
        [rayymnw,rayymni] = min(rayy(:,zlist,:,:,wlist),[],5);
        wmn = zeros(size(rayy(:,zlist,:,:,wlist)));
        wmx = wmn;
        for wm = 1:w0
            wmn(:,:,:,:,wm) = rayymni==wm;
            wmx(:,:,:,:,wm) = rayymxi==wm;
        end
        rayzmxw = sum(rayz(:,zlist,:,:,wlist).*wmx,5);
        rayzmnw = sum(rayz(:,zlist,:,:,wlist).*wmn,5);
        hold on;
        for zp = 1:z0
            for ej = 1:numel(eltinds)
                ncolor = ncolors(glasinds==surfdata{eltinds(ej),8},:);
                elth(ej,zp) = patch(eltz(:,ej)+gthik(eltinds(ej),zlist(zp)),elty(:,ej)+offset(zp),ones(size(eltz(:,ej))),'facecolor',ncolor,'edgecolor','k','LineSmoothing','on','linewidth',0.5);
            end
            for fp = 1:f
                if dcy(zlist(zp),fp) == 0
                    rs = 2:3;
                else
                    rs = 1:3;
                end
                fraymatz = squeeze(rayz(:,zlist(zp),fp,rs,wref)) + repmat(gthik(1:end-1,zlist(zp)),[1,numel(rs)]);
                fraymatz = padarray(fraymatz,[1 0],rayz0,'pre');
                fraymaty = squeeze(rayy(:,zlist(zp),fp,rs,wref)) + offset(zp);
                fraymaty = cat(1,squeeze(rayy0(1,zlist(zp),fp,rs,wref)).'+offset(zp),fraymaty);
                plot(fraymatz,fraymaty,'color',fcolors{fp},'LineSmoothing','on','linewidth',0.5);
            end
            plot(stoppos(zlist(zp)).*ones(2),[stopsd,-stopsd;stopsd+maxsd*stopedge,-(stopsd+maxsd*stopedge)]+offset(zp),'-k','linewidth',2);
        end
        for zp = 1:z0
            for fp = 1:f
                if dcy(zlist(zp),fp) == 0
                    rs = 2:3;
                else
                    rs = 1:3;
                end
                fraypatz = [ squeeze(rayzmxw(:,zp,fp,rs)) + repmat(gthik(1:end-1,zlist(zp)),[1,numel(rs)]) ; flipud(squeeze(rayzmnw(:,zp,fp,rs)) + repmat(gthik(1:end-1,zlist(zp)),[1,numel(rs)])) ];
                fraypatz = padarray(fraypatz,[1 0],rayz0,'both');
                fraypaty = [ squeeze(rayymxw(:,zp,fp,rs)) + offset(zp) ; flipud(squeeze(rayymnw(:,zp,fp,rs)) + offset(zp)) ];
                fraypaty = cat(1,squeeze(max(rayy0(1,zlist(zp),fp,rs,:),[],5)).'+offset(zp), fraypaty, squeeze(min(rayy0(1,zlist(zp),fp,rs,:),[],5)).'+offset(zp));
                patch(fraypatz,fraypaty,zeros(size(fraypaty)),'edgecolor','none','FaceColor',fcolors{fp},'FaceAlpha',0.2);
            end
            plot(stoppos(zlist(zp)).*ones(2),[stopsd,-stopsd;stopsd+maxsd*stopedge,-(stopsd+maxsd*stopedge)]+offset(zp),'-k','linewidth',2);
        end
        rayyref = rayy(:,zlist,:,:,wref);
        [~,maxrayind] = max(abs(rayyref(:)));
        [ssub,zsub,fsub,rsub] = ind2sub(size(rayyref),maxrayind);
        plot(rayz(ssub,zlist(zsub),fsub,rsub,wref)+gthik(ssub,zlist(zsub)),rayy(ssub,zlist(zsub),fsub,rsub,wref)+offset(zsub),'or','markersize',10,'linewidth',2);
        plot([gthik(end,zlist);gthik(end,zlist)],[offset.'+surfdata{end,5};offset.'-surfdata{end,5}],'-k');
        hold off;
        [offsets,offord] = sort(offset);
        set(gca,'ytick',offsets,'yticklabel',round(zefl(offord)));
        grid on;
        ylabel('EFL (mm)');
        xlabel('Position (mm)');
        
        % text axis
        fs = 12; % font size
        ax2 = axes('units','normal','position',[0.75 0.5 .25 0.5],'xlim',[0,1],'ylim',[0 1],'visible','off');
        text(0.15,0.95,sprintf('EFL G%u = %0.1f\n',[1:numgrps;grpefl(:,wref).']),'horizontalalignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        text(0.15,0.6,sprintf('PTB Z%u = %0.4f\n',[zlist(1),zlist(end);sum(dbtho{band}(:,[1,end],4),1)]),'horizontalalignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        text(0.15,0.4,sprintf('FNO Z%u = %0.2f\n',[zlist(1),zlist(end);zfno(band,[1,numel(zlist)])]),'HorizontalAlignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        if (ismember(s-1,zpr) || ismember(s,zpr)) && sum(abs(diff(zthik(s-1,zlist))))>0.01  && sum(abs(diff(zthik(s,zlist))))>0.01
            text(0.15,0.2,sprintf('Length = %0.0f\nDiameter = %0.1f',[max(max(gthik(:,zlist))),maxsd*2]),'HorizontalAlignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        else
            text(0.15,0.2,sprintf('Length = %0.0f\nDiameter = %0.1f\nBFL = %0.1f',[max(max(gthik(:,zlist))),maxsd*2,sum(zthik((s-1):s,zlist(1)))]),'HorizontalAlignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        end
        stopgrp = find(stopsurf>=gbnums(:,1) & stopsurf<=gbnums(:,2));
        if ~isempty(stopgrp)
            text(0.15,0.01,sprintf('Stop @Group#%u',stopgrp),'HorizontalAlignment','left','VerticalAlignment','top','fontsize',fs,'fontweight','bold');
        end
        line('xdata',0.1,'ydata',0.115,'marker','o','color','r','markersize',10,'linewidth',2);
        
        % zoom motion axes
        ax3 = axes('units','normal','position',[0.8 0.08 0.18 0.35]);
        [zeflsort,zsrt] = sort(zefl);
        gs = gthik(gbnums(:,1),zlist(zsrt));
        ge = gthik(gbnums(:,2),zlist(zsrt));
        grppos = (ge+gs)./2;
        set(ax3,'YDir','reverse');
        hold on;
        for gj = 1:numgrps
            patch(round([zeflsort,fliplr(zeflsort)]),[gs(gj,:),fliplr(ge(gj,:))],zeros(1,2*z0),'FaceColor',gcolorss(gj,:),'EdgeColor',gcolors(gj,:),'LineSmoothing','on','facealpha',0.8);
            plot(round(zeflsort),grppos(gj,:),'.','color',gcolors(gj,:),'markersize',10);
        end
        ihan = plot(round(zeflsort),gthik(end,zlist(zsrt)),'-k'); % image plane position
        plot(round(zeflsort),gthik(stopsurf,zlist(zsrt)),'color',0.2.*[1 1 1],'linewidth',2);
        hold off;
        title('Zoom Motion');
        xlabel('EFL (mm)');
        ylabel('Group Position (mm)');
        set(ax3,'xtick',round(zeflsort),'xlim',[round(zeflsort(1)),round(zeflsort(end))]);
        
        % recolor view zoom lens plot and create legend
        eltgrp = arrayfun(@(eltsurf) find(eltsurf>=gbnums(:,1) & eltsurf<=gbnums(:,2),1), surfnums(eltinds));
        grpnames = cell(size(gbnums,1),1);
        for gci = 1:numgrps
            set(elth(eltgrp==gci,:),'facecolor',gcolorss(gci,:));
            grpnames{gci} = ['Group ' num2str(gci)];
        end
        f1l = legend(elth(diff([eltgrp;0])~=0),grpnames,'location','southoutside','orientation','horizontal','parent',f1h,'box','off');
        set(f1l, 'units', 'normal');
        lposi = get(f1l, 'position');
        set(f1l, 'position', [lposi(1) 0.03 lposi(3:4)]);
    end

%% Groups Plot

figh = initfig('Groups',3);
panh = uipanel(figh,'units','normal','position',[0 0 .6 1],'bordertype','none','backgroundcolor','w');
uicontrol(panh,'style','text','string',[dbname{1} ', ' dbname{2}],'units','normal','position',[0 .96 1 .04],'fontweight','bold','fontsize',12,'backgroundcolor','w');
suhan = zeros(numgrps,1);
elthan = zeros(numel(eltinds),1);
glasi = elthan;
for gi = 1:numgrps
    suhan(gi) = subplot_tight(sdim(1),sdim(2),gi,0.08,'parent',panh,'box','on');
    geltindsi = find(eltinds>=gbnums(gi,1) & eltinds<=gbnums(gi,2));
    geltinds = eltinds(geltindsi);
    eltg = zeros(size(eltinds));
    gglasinds = unique(cell2mat(surfdata(geltinds,8)));
    gfirelt = zeros(size(gglasinds));
    for ii = 1:numel(gglasinds);
        gfirelt(ii) = find(cell2mat(surfdata(geltinds,8))==gglasinds(ii),1);
    end
    
    if stopsurf>=surfnums(min(geltinds)) && stopsurf<=surfnums(max(geltinds))
        hold on;
        stoph = plot(stoppos(1).*ones(2)-gthik(geltinds(1)),[stopsd,-stopsd;stopsd+stopsd*stopedge,-(stopsd+stopsd*stopedge)],'-k','linewidth',2);
        hold off;
    end
    
    for ei = 1:numel(geltinds)
        geltz = eltz(:,find(eltinds==geltinds(ei),1));
        gelty = elty(:,find(eltinds==geltinds(ei),1));
        gncolor = ncolors(glasinds==surfdata{geltinds(ei),8},:);
        eltg(ei) = patch(geltz+gthik(geltinds(ei))-gthik(geltinds(1)),gelty,ones(size(geltz)),'facecolor',gncolor,'edgecolor','k','LineSmoothing','on','facealpha',0.9,'buttondownfcn',@selectelt,'hittest','on');
        elthan(geltindsi(ei),1) = eltg(ei);
        glasi(geltindsi(ei),1) = find(glasinds==surfdata{geltinds(ei),8});
        h = get(gca,'children');
        set(gca,'children',[setdiff(h,eltg(ei));eltg(ei)]);
        xpos = (geltz(1)+geltz(end))/2+gthik(geltinds(ei))-gthik(geltinds(1));
%         eltpower = eltpow(find(eltinds==geltinds(ei),1));
        eltfoclen = eltefl(find(eltinds==geltinds(ei),1));
        if ei == 1
%             labpow = text(min(geltz(:))+gthik(geltinds(ei))-gthik(geltinds(1)),0,['\Phi=', sprintf('%0.1f',eltpower)],'VerticalAlignment','middle','HorizontalAlignment','right','BackgroundColor','w','fontweight','bold');
            labefl = text(min(geltz(:))+gthik(geltinds(ei))-gthik(geltinds(1)),0,['f=', sprintf('%0.0f',eltfoclen)],'VerticalAlignment','middle','HorizontalAlignment','right','BackgroundColor','w','fontweight','bold');
        elseif ei == numel(geltinds);
%             labpow = text(max(geltz(:))+gthik(geltinds(ei))-gthik(geltinds(1)) + 0.05*maxsd,0,['\Phi=', sprintf('%0.1f',eltpower)],'VerticalAlignment','middle','HorizontalAlignment','left','BackgroundColor','w','fontweight','bold');
            labefl = text(max(geltz(:))+gthik(geltinds(ei))-gthik(geltinds(1)) + 0.05*maxsd,0,['f=', sprintf('%0.0f',eltfoclen)],'VerticalAlignment','middle','HorizontalAlignment','left','BackgroundColor','w','fontweight','bold');
        elseif ei/2 ~= round(ei/2)
%             labpow = text(xpos,gelty(1)*1.1,['\Phi=', sprintf('%0.1f',eltpower)],'VerticalAlignment','top','HorizontalAlignment','center','BackgroundColor','w','fontweight','bold');
            labefl = text(xpos,gelty(1)*1.1,['f=', sprintf('%0.0f',eltfoclen)],'VerticalAlignment','top','HorizontalAlignment','center','BackgroundColor','w','fontweight','bold');
        elseif ei/2 == round(ei/2)
%             labpow = text(xpos,gelty(numpts)*1.1,['\Phi=', sprintf('%0.1f',eltpower)],'VerticalAlignment','bottom','HorizontalAlignment','center','BackgroundColor','w','fontweight','bold');
            labefl = text(xpos,gelty(numpts)*1.1,['f=', sprintf('%0.0f',eltfoclen)],'VerticalAlignment','bottom','HorizontalAlignment','center','BackgroundColor','w','fontweight','bold');
        end
        if eltfoclen<0
%             set(labpow,'color','r');
            set(labefl,'color','r');
        end
    end
    % draw cemented surface indicators
    if any(diff(geltinds)==1)
        firsts = geltinds(diff(geltinds)==1);
        cemh = zeros(size(firsts));
        for fi = 1:numel(firsts);
            surf1 = elty(numpts+1:end,eltinds==(firsts(fi)));
            surf2 = elty(1:numpts,eltinds==firsts(fi)+1);
            if max(surf1)<max(surf2)
                surfy = surf1;
                surfz = eltz(numpts+1:end,eltinds==firsts(fi));
                offsetz = gthik(firsts(fi))-gthik(geltinds(1));
            else
                surfy = surf2;
                surfz = eltz(1:numpts,eltinds==(firsts(fi)+1));
                offsetz = gthik(firsts(fi)+1)-gthik(geltinds(1));
            end
            cemh(fi) = line(surfz+offsetz,surfy,'color',[0.3 0.3 0.3],'linewidth',2.5,'linesmoothing','on');
        end
        uistack(cemh,'top');
    end
    axis equal;
    buff = 0.3; % buffer fraction
    xlo = get(gca,'xlim');
    xln = xlo + [-diff(xlo)*buff,diff(xlo)*buff];
    ylo = get(gca,'ylim');
    yln = ylo + [-diff(ylo)*buff,diff(ylo)*buff];
    hold on;
    plot(xln([1;2;2;1]),yln([1;1;2;2]),'.w');
    hold off;
    grid on;
    title(['Group ' num2str(gi) '  (EFL = ' sprintf('%0.0fmm, %0.0fmm)', grpefl(gi,dbwref))],'fontweight','bold');
end

% create glass info table
legnams = surfdata(eltinds(firelts),6);
legcats = surfdata(eltinds(firelts),9);
ns = index_w(eltinds(firelts),dbwref);
vs = abbe(eltinds(firelts),:);
ps = partial(eltinds(firelts),:);

xspac = [0.5, 1.2, 1, 0.6, 0.6, 0.6]; % relative widths for columns [ patch, name, catalog, index, abbe, partial ]
xbnd = [0,cumsum(xspac)];
ybnd = 0.5:(numel(glasinds)+0.5);
axtab = axes('parent',figh,'units','normal','position',[0.58 0.01 0.41 0.89],'xlim',[min(xbnd) max(xbnd)],'ylim',[0.5, numel(glasinds)+0.5],'xtick',xbnd,'ytick',ybnd,...
    'xticklabel',repmat({''},1,4),'yticklabel',repmat({''},1,numel(glasinds)+1),'ticklength',[0 0],'box','off','xgrid','on','ygrid','on','gridlinestyle','-','ydir','reverse','layer','top');
set(axtab,'DefaulttextHorizontalAlignment','left','DefaulttextVerticalAlignment','middle','DefaulttextFontWeight','bold');
% text(0.5,0.5,['\lambda_{ref} = ' sprintf('%0.0f nm',wav(wref))],'fontweight','bold','fontsize',12,'horizontalalignment','center','verticalalignment','middle');
patx = [xbnd(1)+.1*diff(xbnd(1:2)); xbnd(2)-.1*diff(xbnd(1:2)); xbnd(2)-.1*diff(xbnd(1:2)); xbnd(1)+.1*diff(xbnd(1:2))];
bandcolors = min(1,pancol+0.9);
leglash = zeros(numel(glasinds),1);
for gl = 1:numel(glasinds)
    leglash(gl) = patch(patx,[gl+.4;gl+.4;gl-.4;gl-.4],zeros(4,1),'facecolor',ncolors(gl,:),'edgecolor','k','facealpha',0.9,'buttondownfcn',@selectglas);
    text(xbnd(2)+.05*diff(xbnd(2:3)),gl,legnams{gl}); % glass name column
    text(xbnd(3)+.1*diff(xbnd(3:4)),gl,legcats{gl},'fontweight','normal'); % glass catalog column
    for bi = 1:dbnum
        offs = -0.25;
        if bi==2
            offs = -offs;
        end
        patch(xbnd([4;7;7;4]),gl+[-0.5;-0.5;0;0]+(bi-1)*0.5,zeros(4,1),'facecolor',bandcolors(bi,:),'edgecolor','none');
        text(mean(xbnd(4:5)),gl+offs,sprintf('%0.2f',ns(gl,bi)),'HorizontalAlignment','center')%,'backgroundcolor',bandcolors(bi,:));
        text(mean(xbnd(5:6)),gl+offs,sprintf('%0.1f',vs(gl,bi)),'HorizontalAlignment','center')%,'backgroundcolor',bandcolors(bi,:));
        text(mean(xbnd(6:7)),gl+offs,sprintf('%0.3f',ps(gl,bi)),'HorizontalAlignment','center')%,'backgroundcolor',bandcolors(bi,:));
    end
end
tf = 0.5-0.0077*numel(glasinds);
text(mean(xbnd(2:3)),tf,'Name','HorizontalAlignment','center','VerticalAlignment','bottom');
text(mean(xbnd(3:4)),tf,'Catalog','HorizontalAlignment','center','VerticalAlignment','bottom');
text(mean(xbnd(4:5)),tf,'Index','HorizontalAlignment','center','VerticalAlignment','bottom');
text(mean(xbnd(5:6)),tf,'Abbe','HorizontalAlignment','center','VerticalAlignment','bottom');
text(mean(xbnd(6:7)),tf,'Partial','HorizontalAlignment','center','VerticalAlignment','bottom');
wavtxt = cell(2,1);
for bi = 1:dbnum
    if dbwref(bi) == wshort(bi)
        wavtxt{bi} = sprintf('\\bf%0.0fnm\\rm, %0.0fnm, %0.0fnm',wav([wshort(bi),wmid(bi),wlong(bi)]));
    elseif dbwref(bi) == wlong(bi)
        wavtxt{bi} = sprintf('%0.0fnm, %0.0fnm, \\bf%0.0fnm\\rm',wav([wshort(bi),wmid(bi),wlong(bi)]));
    elseif dbwref(bi) == wmid(bi)
        wavtxt{bi} = sprintf('%0.0fnm, \\bf%0.0fnm\\rm, %0.0fnm',wav([wshort(bi),wmid(bi),wlong(bi)]));
    else
        wavtxt{bi} = sprintf('\\bf%0.0fnm\\rm, %0.0fnm, %0.0fnm, %0.0fnm',wav([dbwref(bi),wshort(bi),wmid(bi),wlong(bi)]));
    end
end
for bi = 1:dbnum
    text(mean(xbnd(1:7)),tf-0.035*numel(glasinds)*(dbnum+1-bi),[dbname{bi} ': ' wavtxt{bi}],'HorizontalAlignment','center',...
        'VerticalAlignment','bottom','fontweight','normal','edgecolor',[0.5 0.5 0.5]+0.5,'backgroundcolor',bandcolors(bi,:));
end

%% THO Plot
% plot of third order aberrations
figtho = initfig('Third Order Aberrations',4);
thonames = {'SA';'TCO';'TAS';'PTB';'DST';'AX';'LAT'};
sptho = cell(numgrps+1,1);
sptho(:) = {{['-g']}};
[spphan,labelfontsize] = subplotplus({sptho,sptho});
spphan = reshape(spphan,numgrps+1,[]);
grmax = zeros(numgrps,dbnum);
grmin = grmax;
gmid = grmax;
totbar = cell(dbnum,1);
rng = zeros(dbnum,1);
biggest = rng;
for bi = 1:dbnum
    tho = dbtho{bi};
    grmax(:,bi) = max(max(tho,[],3),[],2);
    grmin(:,bi) = min(min(tho,[],3),[],2);
    gmid(:,bi) = (grmax(:,bi)+grmin(:,bi))./2;
    totbar{bi} = transpose(squeeze(sum(tho,1)));
    [rng0,biggest0] = max([grmax(:,bi)-grmin(:,bi);(max(totbar{bi}(:))-min(totbar{bi}(:)))]);
    rng(bi) = rng0/2;
    biggest(bi) = biggest0;
end
grplab = arrayfun(@(gc) ['Group ' int2str(gc)],(1:numgrps).','UniformOutput',false);
for bi = 1:dbnum
    for gc = 1:numgrps
        set(gcf,'CurrentAxes',spphan(gc,bi));
        barhan = bar(transpose(squeeze(dbtho{bi}(gc,:,:))));
        for bh = 1:numel(barhan)
            set(barhan(bh),'facecolor',gcolors(bh,:));
        end
        ylabel(grplab{gc},'FontSize',10,'fontweight','bold');
        ylim([gmid(gc,bi)-max(rng), gmid(gc,bi)+max(rng)]);
        if gc==1
            set(spphan(gc,bi),'XAxisLocation','top','xticklabel',thonames);
            title([dbname{bi} ', \lambda_{ref} = ' sprintf('%0.0f nm',wav(dbwref(bi)))],'Fontsize',12,'FontWeight','bold');
        end
    end
    
    set(gcf,'CurrentAxes',spphan(numgrps+1,bi));
    barhan = bar(totbar{bi});
    for bh = 1:numel(barhan)
        set(barhan(bh),'facecolor',gcolors(bh,:));
    end
    ylabel('Total','FontSize',10,'fontweight','bold');
    tmid = (max(totbar{bi}(:)) + min(totbar{bi}(:)))/2;
    ylim([tmid-max(rng), tmid+max(rng)]);
    
    % set ytick labels to be the same on all graphs
    yspac = mean(diff(get(spphan(biggest(bi),bi),'ytick')));
    ytik = -10*yspac:yspac:10*yspac;
    set(spphan(:,bi),'ytick',ytik,'ygrid','on','fontweight','bold');
    
    set(spphan(numgrps+1,bi),'xticklabel',thonames);
    leglab1 = arrayfun(@(zpos) sprintf('Z%0.0f, %0.0fmm',zpos,eflwz(dbwref(bi),zpos)),dbzoomi{bi},'UniformOutput',false);
    opos = get(spphan(numgrps+1,bi),'position');
%     lega = legend(leglab1,'location','SouthOutside','orientation','horizontal');
    set(spphan(numgrps+1,bi),'position',opos);
%     set(lega,'units','pixels','position',[5+960/2*(bi-1), 10, 960/2-10, 25]);
    pw = 0.1;
    ph = 0.7;
    axes('box','on','xtick',[],'ytick',[]','xlim',[0.9,numel(barhan)+1],'ylim',[-1,1],'units','pixels','position',[5+960/2*(bi-1), 10, 960/2-10, 22]);
    for bh = 1:numel(barhan)
        patch([bh;bh+pw;bh+pw;bh],[ph;ph;-ph;-ph],[0;0;0;0],'facecolor',gcolors(bh,:),'edgecolor','k');
        text(bh+pw+0.05,0,leglab1{bh},'HorizontalAlignment','left','VerticalAlignment','middle');
    end
end

% THO Group Balancing Matrix
figbal = initfig('Group Balancing Matrix',5);
gbmd = 450;% group balancing matrix dimensions in pixels
thobalog = cell(2,1);
for bi = 1:dbnum
    thobal = zeros(numgrps,numgrps,numel(thonames));
    for gba = 1:numgrps
        thobal(:,gba,:) = sum(abs(repmat(dbtho{bi}(gba,:,:),[numgrps,1,1]) + dbtho{bi}), 2);
    end
    % thobal = thobal.*repmat((eye(numgrps).*(-0.5)+1),[1,1,numel(thonames)]);
    thobalog1 = thobal==repmat(min(thobal,[],1),[numgrps,1,1]);
    thobalog2 = thobal==repmat(min(thobal,[],2),[1,numgrps,1]);
    thobalog{bi} = thobalog1&thobalog2;
    thobalog{bi} = logical(thobalog{bi}.*repmat(1-(sum(thobalog{bi},1)==numgrps),[numgrps,1,1]));
end
thobalogboth = thobalog{1} & thobalog{2};

for bi = 1:dbnum
    tho = dbtho{bi};
    axes('units','pixels','position',[960/2*(1+(bi-1))-gbmd-5, 630/2-gbmd/2, gbmd, gbmd]);
    set(gca,'Ticklength',[0 0]);
    axis square; grid off;
    title(['Group Balancing Matrix: ' dbname{bi}],'Fontsize',12,'FontWeight','bold');
    xlim([0.5 numgrps+0.5]);
    ylim([0.5 numgrps+0.5]);
    set(gca,'xtick',1:numgrps,'ytick',1:numgrps,'xminortick','off','yminortick','off','xticklabel',grplab,'yticklabel',[],'ydir','reverse','XAxisLocation','top','fontweight','bold');
%     thobal = zeros(numgrps,numgrps,numel(thonames));
%     for gba = 1:numgrps
%         thobal(:,gba,:) = sum(abs(repmat(tho(gba,:,:),[numgrps,1,1]) + tho), 2);
%     end
%     % thobal = thobal.*repmat((eye(numgrps).*(-0.5)+1),[1,1,numel(thonames)]);
%     thobalog1 = thobal==repmat(min(thobal,[],1),[numgrps,1,1]);
%     thobalog2 = thobal==repmat(min(thobal,[],2),[1,numgrps,1]);
%     thobalog = thobalog1&thobalog2;
%     thobalog = logical(thobalog.*repmat(1-(sum(thobalog,1)==numgrps),[numgrps,1,1]));
    for gbr = 1:numgrps
        text(.5,gbr,grplab{gbr},'HorizontalAlignment','center','VerticalAlignment','bottom','rotation',90,'fontweight','bold');
        for gbc = 1:numgrps
            if gbc<=gbr
                patch([gbc-0.5;gbc-0.5;gbc+0.5;gbc+0.5],[gbr-0.5;gbr+0.5;gbr+0.5;gbr-0.5],[1;1;1;1],'edgecolor','k','facecolor',[1 1 1].*0.9);
            end
            if any(thobalog{bi}(gbr,gbc,:))
                alist = thonames(thobalog{bi}(gbr,gbc,:));
                blist = thobalogboth(gbr,gbc,thobalog{bi}(gbr,gbc,:));
                atext = '';
                for at = 1:numel(alist)
                    if at == numel(alist)
                        if blist(at)
                            atext = [atext '\bf\color[rgb]{0.4 0 0}' alist{at}];
                        else
                            atext = [atext alist{at}];
                        end
                    else
                        if blist(at)
                            atext = [atext '\bf\color[rgb]{0.4 0 0}' sprintf([alist{at} '\n']) '\rm\color{black}'];
                        else
                            atext = [atext sprintf([alist{at} '\n'])];
                        end
                    end
                end
                text(gbc,gbr,atext,'HorizontalAlignment','center','VerticalAlignment','middle','fontsize',9,'interpreter','tex');
            end
            if gbr==1
                line([gbc+0.5;gbc+0.5],[0.5 numgrps+0.5],'color','k');
            end
            if gbc==1
                line([0.5 numgrps+0.5],[gbr+0.5;gbr+0.5],'color','k');
            end
        end
    end
end

%% RIM Plot
% two figures of rim plots, one for each band
% turn into function and run twice as done with lens drawings
% plot RIM data
centind = ceil(rimpts/2);
for bi = 1:dbnum
    rimplot(bi,dbzoomi{bi},dbzrim{bi},dbwavi{bi},rngr);
end

    function rimplot(band,zlist,zrim,wlist,rngr)
        w0 = numel(wlist);
        axlinwid = 1;
        figr = initfig(['RIM Plot ' int2str(band)],band+5,dbname{band});
        zmodule = repmat({{['-g']},{['-g']}},f,1);
        rimyoffset = repmat(rimy(centind,dbwref(band),:,zlist),[rimpts,w0,1,1]);
        rimxoffset = repmat(rimx(centind,dbwref(band),:,zlist),[rimpts,w0,1,1]);
        rimyc = rimy(:,wlist,:,zlist)-rimyoffset;
        rimxc = rimx(:,wlist,:,zlist)-rimxoffset;
        if rngr == 0
            rngr1 = 1;
        else
            rngr1 = rngr;
        end
        
        spphr = subplotplus({repmat(zmodule,1,numel(zrim))});
        
%         leglab2 = arrayfun(@(wl) [sprintf('%0.2f',wl) ' \mum'], wav(wlist)./1000, 'UniformOutput', false); % in microns
        leglab2 = arrayfun(@(wl) sprintf('%0.0f nm',wl), wav(wlist), 'UniformOutput', false);
        leglab3 = arrayfun(@(zpos) sprintf('Z%0.0f,  EFL %0.0fmm',zpos,eflwz(dbwref(band),zpos)),zlist,'UniformOutput',false);
        set(spphr,'units','normal');
        rimax = zeros(f,numel(zrim));
        linh = zeros(f,numel(zrim),2); % axis line handles [field, zoom, tan|sag]
        txth = linh; % upper and lower limit text labels [field, zoom, upper|lower]
        tith = zeros(1,numel(zrim));
        for fd = 1:f
            for zd = 1:numel(zrim)
                subind = (f-fd)*numel(zrim)*2 + (zd-1)*2 + 1;
                ypos0 = get(spphr(subind),'position');
                ypos0(3) = ypos0(3)*2*2/3;
                ypos0(1) = ypos0(1)+(zd-1)*0.1/numel(zrim);
                xpos0 = get(spphr(subind+1),'position');
                xpos0(3) = xpos0(3)*2*1/3;
                xpos0(1) = ypos0(1)+ypos0(3);
                set(spphr(subind),'xlim',[-1.05 1.05],'ylim',[-rngr1 rngr1].*1.1,'position',ypos0,'box','off','xtick',[],'ytick',[],'yticklabel',[],'visible','off');
                set(spphr(subind+1),'xlim',[-0.02 1.05],'ylim',[-rngr1 rngr1].*1.1,'position',xpos0,'box','off','xtick',[],'ytick',[],'yticklabel',[],'visible','off');
                % tangential axis
                set(gcf,'CurrentAxes',spphr(subind));
                set(gca,'colororder',wcolors);
                linh0 = line([-1,0;1,0],[0,-rngr1;0,rngr1],'color','k','linewidth',axlinwid);
                linh(fd,zd,1) = linh0(2);
                pyhi = 1-vig(fd,zlist(zrim(zd)),1);
                pylo = -1+vig(fd,zlist(zrim(zd)),2);
                yfind = pupy>pylo & pupy<pyhi;
                puppy = [pylo;pupy(yfind);pyhi];
                newends = zeros(2,w0);
                for wy = 1:w0
                    newends(:,wy) = interp1(pupy,rimyc(:,wy,fd,zrim(zd)),[pylo;pyhi]);
                end
                rimpy = [newends(1,:); rimyc(yfind,:,fd,zrim(zd)); newends(2,:)];
                line(repmat(puppy,1,w0),rimpy,'linesmoothing','on','linewidth',1);
                % label y axis extremes
                txth(fd,zd,1) = text(0.5,rngr1,num2str(rngr1),'HorizontalAlignment','center','VerticalAlignment','top','fontsize',8);
                txth(fd,zd,2) = text(0.5,-rngr1,num2str(-rngr1),'HorizontalAlignment','center','VerticalAlignment','bottom','fontsize',8);
                grid off;
                if zd==1
                    text(-1.2,0,sprintf('Field %0.0f\n( %0.2f rel. field )',fd,abs(yan(fd))/max(abs(yan))),'fontsize',12,'FontWeight','bold','HorizontalAlignment','center','VerticalAlignment','bottom','rotation',90);
                end
                if fd==f
                    tith(zd) = text(0.5,rngr1*1.2,leglab3{zrim(zd)},'fontsize',12,'FontWeight','bold','HorizontalAlignment','center','VerticalAlignment','bottom');
                end
                % saggital axis
                set(gcf,'CurrentAxes',spphr(subind+1));
                set(gca,'colororder',wcolors);
                linh0 = line([-1,0;1,0],[0,-rngr1;0,rngr1],'color','k','linewidth',axlinwid);
                linh(fd,zd,2) = linh0(2);
                pxhi = 1-vig(fd,zlist(zrim(zd)),3);
                pxlo = -1+vig(fd,zlist(zrim(zd)),4);
                xfind = pupx>pxlo & pupx<pxhi;
                puppx = [pxlo;pupx(xfind);pxhi];
                newends = zeros(2,w0);
                for wx = 1:w0
                    newends(:,wx) = interp1(pupx,rimxc(:,wx,fd,zrim(zd)),[pxlo;pxhi]);
                end
                rimpx = [newends(1,:); rimxc(xfind,:,fd,zrim(zd)); newends(2,:)];
                line(repmat(puppx,1,w0),rimpx,'linesmoothing','on','linewidth',1);
                grid off;
%                 if fd==1 && zrim(zd)==midz
%                     opos = get(spphr(subind+1),'position');
%                     legb = legend(lnh,leglab2,'location','SouthOutside','orientation','horizontal','fontsize',12,'fontweight','bold');
%                     set(spphr(subind+1),'position',opos);
%                     set(legb,'units','pixels');
%                     opos = get(legb,'position');
%                     opos(2) = 10;
%                     opos(3) = 700;
%                     opos(1) = 960/2-opos(3)/2;
%                     set(legb,'position',opos,'box','off');
%                 end
                pw = 0.15;
                ph = 0.7;
                basewid = 120;
                axes('box','on','xtick',[],'ytick',[],'xlim',[0.7,w0+0.7],'ylim',[-1,1],'units','pixels','position',[max(5,(960-basewid*w0)/2) 10 min(960-10,basewid*w0) 25]);
                for wh = 1:w0
                    line(wh+linspace(-pw,0,11).',ph.*(linspace(-1,1,11).').^3,'color',wcolors(wh,:),'linesmoothing','on');%
                    text(wh+0.05,0,leglab2{wh},'HorizontalAlignment','left','VerticalAlignment','middle','fontsize',12,'fontweight','bold');
                end
                rimax(fd,zd) = max(max(abs(rimpy(:))),max(abs(rimpx(:))));
            end
        end
        
        if rngr == 0
%             sortrim = sort(abs([rimxc(:);rimyc(:)]));
%             rngr = sortrim(round(numel(sortrim)*1.00));
            rngr = max(rimax(:));
            rord = order(rngr); % order of magnitude of the range
            rngr = ceil(10^(-rord)*rngr*2)/10^(-rord)/2; % round range to one value in highest order of magnitude
            set(spphr,'ylim',[-rngr rngr].*1.1);
            set(linh,'ydata',[-rngr;rngr]);
            set(txth(:,:,1),'position',[0.5 rngr],'string',num2str(rngr));
            set(txth(:,:,2),'position',[0.5 -rngr],'string',num2str(-rngr));
            set(tith,'position',[0.5 rngr*1.2]);
        end
        
        % "Scale" text axis
        axes('units','normal','position',[0 .95 1 0.05],'visible','off','xlim',[0,1],'ylim',[0 1],'parent',figr);
        text(0.5,0.5,[dbname{band} ' Ray Aberrations, Scale: \pm ' sprintf('%0.0f',rngr*1000) ' \mum'],'HorizontalAlignment','center','VerticalAlignment','middle','fontsize',12,'fontweight','bold');
    end

%% MTF Plot
% plot of MTF data
for bi = 1:dbnum
    mtfplot(bi,fliplr(dbzrim{bi}),dbzoomi{bi},dbwavi{bi});
end

    function mtfplot(band,zrimm,zlist,wlist)
        figm = initfig(['MTF ' int2str(band)],band+7,dbname{band});
        spm = zeros(numel(zrimm),1);
        mbot = ((1:numel(zrimm))-1)./numel(zrimm);
        mspac = 1/numel(zrimm);
        margl = 0.08; % left margin
        margb = 0.05; % bottom margin
        margt = 0.01; % top margin
        mwid = 0.5; % mtf plot width
        taxis = axes('units','normal','position',[margl+mwid,0,1-margl-mwid,1],'xlim',[0 1],'ylim',[0 1],'visible','off');
        leglab1 = arrayfun(@(zpos) sprintf('Z%0.0f, %0.0fmm',zpos,eflwz(dbwref(band),zpos)),zlist,'UniformOutput',false);
        for mi = 1:numel(zrimm)
            spm(mi) = axes('units','normal','position',[margl,margb+mbot(numel(zrimm)-mi+1),mwid,mspac-margb-margt],'xlim',[0,maxfreq{band}(zrimm(mi))],'ylim',[0 1],'fontsize',12,'colororder',fcolorsrgb);
            ylabel(leglab1{zrimm(mi)},'fontweight','bold');
            grid on;
            tanlines = line(squeeze(mtfdata{band}(:,1,:,numel(zrimm)+1-mi)),squeeze(mtfdata{band}(:,6,:,numel(zrimm)+1-mi)),'linestyle','-','linesmoothing','on');
            radlines = line(squeeze(mtfdata{band}(:,1,:,numel(zrimm)+1-mi)),squeeze(mtfdata{band}(:,5,:,numel(zrimm)+1-mi)),'linestyle','--');
            dline = line(squeeze(mtfdata{band}(:,1,1,numel(zrimm)+1-mi)),squeeze(mtfdata{band}(:,2,1,numel(zrimm)+1-mi)),'linestyle','-.','color','k');
            ltick = get(gca,'xtick');
            ltick = unique([ltick,maxfreq{band}(zrimm(mi))]);
            set(gca,'xtick',ltick);
%             text(0.06,margb+mbot(numel(zrimm)-mi+1)-0.005,'L/mm','parent',taxis,'HorizontalAlignment','left','VerticalAlignment','top','fontsize',12);
            text(1.04*maxfreq{band}(zrimm(mi)),0,'L/mm','HorizontalAlignment','left','VerticalAlignment','top','fontsize',12);
        end
        legmhans = zeros(f*2,1);
        legmlabs = cell(f*2,1);
        legmlabs{1} = 'F1 Diff. Limit (formula)';
 %%%%%%%%%%%%%%%%%%%%%%%%% make into wavelength weight matrix
        for fl = 1:f
            if yan(fl)==0
                legmhans(2*(fl-1)+1) = dline;
                legmhans(2*(fl-1)+2) = tanlines(fl);
                legmlabs{2*(fl-1)+1} = sprintf('F%0.0f:  Diff. Limit (formula)',fl);
                legmlabs{2*(fl-1)+2} = sprintf('F%0.0f:  %0.2f rel. field',fl,0);
            else
                legmhans(2*(fl-1)+1) = tanlines(fl);
                legmhans(2*(fl-1)+2) = radlines(fl);
                legmlabs{2*(fl-1)+1} = sprintf('F%0.0f: T  %0.2f rel. field',fl,abs(yan(fl))/max(abs(yan)));
                legmlabs{2*(fl-1)+2} = sprintf('F%0.0f: R  %0.2f rel. field',fl,abs(yan(fl))/max(abs(yan)));
            end
        end
        opos = get(spm(end),'position');
        legm = legend(legmhans,legmlabs,'location','eastoutside','box','off','fontweight','bold');
        set(spm(end),'position',opos);
        lpos = get(legm,'position');
        set(legm,'units','normal','position',[0.7,0.5,lpos(3:4)]);
        set(gcf,'currentaxes',taxis);
        text(0.5,0.85,['"' dbname{band} '" MTF'],'HorizontalAlignment','center','fontsize',14,'fontweight','bold');
        wavt = 'Wavelength (nm)';
        wavt = [wavt,sprintf('\n%0.0f',wav(wlist))];
        witt = ['Weight ( Z' sprintf([repmat('%0.0f, ',1,numel(zrimm)-1) '%0.0f )'],zlist(zrimm))];
        witt = [witt,sprintf(['\n' repmat('%0.1f, ',1,numel(zrimm)-1) '%0.1f'],wtw(wlist,zlist(zrimm)).')];
        text(0.33,0.25,wavt,'HorizontalAlignment','center','VerticalAlignment','middle','fontsize',12,'fontweight','bold');
        text(0.75,0.25,witt,'HorizontalAlignment','center','VerticalAlignment','middle','fontsize',12,'fontweight','bold');
        set(taxis,'position',[margl+mwid,0,1-margl-mwid,1]);
    end

%% Nested Functions
% "Continue" button callback
    function qin(buth,~,qhan)
        set(get(buth,'parent'),'visible','off');
        drawnow;
        dbzrim(:) = {[]};
        for bj = 1:dbnum
            for zz = 1:numel(dbzoomi{bj})
                if (get(qhan{bj}(zz,1),'Value') == get(qhan{bj}(zz,1),'Max'))
                    dbzrim{bj} = [dbzrim{bj},zz];
                end
                maxfreq{bj}(zz) = str2double(get(qhan{bj}(zz,2),'string'));
            end
            if isempty(dbzrim{bj})
                dbzrim{bj} = 1;
            end
        end
        delete(get(buth,'parent'));
        cancelq = 0;
    end

% element highlight callback
    function selectelt(src,~)
        eltnumber = find(elthan==src);
        if isequal(get(elthan(eltnumber),'edgecolor'), [1 0 0]);
            set(elthan,'edgecolor',[0 0 0],'linewidth',0.5);
            set(leglash,'edgecolor',[0 0 0],'linewidth',0.5);
        else
            set(elthan,'edgecolor',[0 0 0],'linewidth',0.5);
            set(leglash,'edgecolor',[0 0 0],'linewidth',0.5);
            eltnumbers = glasi==glasi(eltnumber);
            set(elthan(eltnumbers),'edgecolor',[1 0 0],'linewidth',1.5);
            set(leglash(glasi(eltnumber)),'edgecolor',[1 0 0],'linewidth',1.5);
        end
    end

% glass highlight callback
    function selectglas(src,~)
        glasnumber = find(leglash==src);
        if isequal(get(leglash(glasnumber),'edgecolor'), [1 0 0]);
            set(elthan,'edgecolor',[0 0 0],'linewidth',0.5);
            set(leglash,'edgecolor',[0 0 0],'linewidth',0.5);
        else
            set(elthan,'edgecolor',[0 0 0],'linewidth',0.5);
            set(leglash,'edgecolor',[0 0 0],'linewidth',0.5);
            eltnumbers = glasi==glasnumber;
            set(elthan(eltnumbers),'edgecolor',[1 0 0],'linewidth',1.5);
            set(leglash(glasnumber),'edgecolor',[1 0 0],'linewidth',1.5);
        end
    end

% update dual band figure function
    function dbupdate(col)
        switch col
            case 0 % band names
                set(dbhant(1,:),'string',dbname{1});
                set(dbhant(2,:),'string',dbname{2});
            case 1 % wavelengths (slider value, cutoff edit box, position band name boxes, Wref radio enable and selection)
                set(coslider,'value',-dbcutoff);
                set(cobox,'string',num2str(dbcutoff));
                db1vpos = min(vposw)+(max(vposw)-min(vposw))*(1-((dbcutoff+min(wav))/2-min(wav))/(max(wav)-min(wav)))-10;
                db2vpos = min(vposw)+(max(vposw)-min(vposw))*(1-((dbcutoff+max(wav))/2-min(wav))/(max(wav)-min(wav)))+10;
                set(dbhant(1,1),'position',[hpos(1) db1vpos diff(hpos(1:2))-30 20]);
                set(dbhant(2,1),'position',[hpos(1) db2vpos diff(hpos(1:2))-30 20]);
                set(bg1,'SelectedObject',dbhanw(dbwref(1),1));
                set(bg2,'SelectedObject',dbhanw(dbwref(2),2));
                selold(1) = get(bg1,'SelectedObject');
                selold(2) = get(bg2,'SelectedObject');
                if ~ismember(selold(1),dbhanw(dbwavi{1},1))
                    set(bg1,'SelectedObject',dbhanw(min(dbwavi{1}),1));
                    dbwref(1) = find(dbhanw(:,1)==get(bg1,'SelectedObject'));
                    dbupdatez();
                end
                if ~ismember(selold(2),dbhanw(dbwavi{2},2))
                    set(bg2,'SelectedObject',dbhanw(max(dbwavi{2}),2));
                    dbwref(2) = find(dbhanw(:,2)==get(bg2,'SelectedObject'));
                    dbupdatez();
                end
                set(dbhanw,'enable','off');
                set(dbhanw(dbwavi{1},1),'enable','on');
                set(dbhanw(dbwavi{2},2),'enable','on');
        end
    end

% function to update zoom titles in dual band figure
    function dbupdatez
        for zb = 1:z
            set(dbhanz(zb,1),'string',[sprintf('Z%0.0f',zb), sprintf(', %0.0fmm',eflwz(dbwref(dbz(:,zb)),zb))]);
        end
        set(dbhanztrunc,'value',0);
        set(dbhanztrunc(dbz.'),'value',1);
    end

% DB slider callback
    function coslidercall(src,~)
        dbcutoff = -round(get(src,'value'));
        dbcutoff = max(min(wav)+1,dbcutoff);
        dbcutoff = min(max(wav)-1,dbcutoff);
        dbwavi{1} = find(wav<=dbcutoff);
        dbwavi{2} = find(wav>dbcutoff);
        dbupdate(1);
    end

% DB cutoff edit box callback
    function coboxcall(src,~)
        dbcutoff = str2double(get(src,'string'));
        dbcutoff = max(min(wav)+1,dbcutoff);
        dbcutoff = min(max(wav)-1,dbcutoff);
        dbwavi{1} = find(wav<=dbcutoff);
        dbwavi{2} = find(wav>dbcutoff);
        dbupdate(1);
    end

% DB name edit boxes callback
    function bandnamecall(~,~)
        dbname{1} = get(dbhant(1,1),'string');
        dbname{2} = get(dbhant(2,1),'string');
        dbupdate(0);
    end

% DB buttongroup callback
    function dbwrefcall(~,~)
        dbwref(1) = find(dbhanw(:,1)==get(bg1,'SelectedObject'));
        dbwref(2) = find(dbhanw(:,2)==get(bg2,'SelectedObject'));
        dbupdatez();
    end

% DB zoom checkbox callback
    function dbzoomcall(~,~)
        dbz(:) = false;
        for zdb = 1:z
            if (get(dbhanz(zdb,2),'Value') == get(dbhanz(zdb,2),'Max'))
            dbz(1,zdb) = true;
            end
            if (get(dbhanz(zdb,3),'Value') == get(dbhanz(zdb,3),'Max'))
            dbz(2,zdb) = true;
            end
        end
        dbupdatez();
    end

% DB cancel button callback
    function dbcancall(~,~)
        delete(dbfig);
    end

% DB OK button callback
    function dbokcall(~,~)
        set(dbfig,'visible','off');
        drawnow;
        canceldb = 0;
        delete(dbfig);
    end
%% debug
save_to_base(1);

end

%% Accessory Functions
% outputs points for drawing an element with front vertex at z=0
function [eltz,elty] = eltpts(pf,surfdata,numpts,varargin) % surf = surface number for front of element, varargin = amount to add to max semi-diameter for looks "0" just to flatten edges
pb = pf+1;
rf = surfdata{pf,3};
rb = surfdata{pb,3};
thi = surfdata{pf,4};
sdf = surfdata{pf,5};
sdb = surfdata{pb,5};
if nargin > 3
    sdf = max([sdf,sdb])+varargin{1};
    sdb = sdf;
end
yf = (linspace(-sdf,sdf,numpts)).';
yb = (linspace(sdb,-sdb,numpts)).';
zf = rf-sign(rf).*sqrt(rf^2 - yf.^2);
zb = thi + rb - sign(rb).*sqrt(rb^2 - yb.^2);
eltz = [zf;zb];
elty = [yf;yb];
end

% send eva command to CV and return number (double)
function numout = cvevadbl(evacmd)
global CodeV;
tempchar = invoke(CodeV,'command',['EVA(' evacmd ')']);
qpos1 = strfind(tempchar,'=');
qpos2 = strfind(tempchar,'Command End');
numout = str2double(tempchar(qpos1(1)+1:qpos2(1)-1));
end

% initiate figure
function fhan = initfig(fname,fignum,varargin)
% fname is the figure (base) name
% fignum is the figure number, determines figure screen position
% varargin is optional subtitle to figure name
ss = get(0,'ScreenSize');
fpos = [10+40*(fignum-1),60+10*(fignum-1),960,630];
fogm = findobj('-regexp','Name',fname,'type','figure');
if isempty(fogm);
    fhan = figure(); % create figure
else
    figure(fogm(1));
    fhan = clf('reset'); % clear previous figure
end
set(fhan,'Position',fpos,'color','w','units','pixels','numbertitle','off');
if nargin<3
    set(fhan,'Name',fname);
else
    set(fhan,'Name',[fname,': ',varargin{1}]);
end
fs = get(fhan,'outerposition');
set(fhan,'outerposition',[1+40*(fignum-1), ss(4)-fs(4)-30*(fignum-1), fs(3), fs(4)],'resize','on');
end

% "Cancel" button callback
function cancel(buth,~)
    set(get(buth,'parent'),'visible','off');
    delete(get(buth,'parent'));
end

% check box callback
function callchk(chk,~,hans)
if (get(chk,'Value') == get(chk,'Max'))
    set(hans,'enable','on');
elseif (get(chk,'Value') == get(chk,'Min'))
    set(hans,'enable','off');
end
end

